package com.nisum.service;



import com.nisum.model.Order;
import com.nisum.utils.ClientUtils;
import lombok.extern.slf4j.Slf4j;
import com.nisum.model.Product;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.Map;


@Slf4j
@Service
public class MyServiceImpl implements MyService {


    @Value("${com.server.url}")
    private String url;

    private final WebClient webClient;

    @Autowired
    public MyServiceImpl( WebClient.Builder webClientBuilder) {
        this.webClient = webClientBuilder.build();
    }

    @Override
    public Flux<Order> getAllOrdersFromService() {
        return this.webClient.get().uri(url+ ClientUtils.ORDERS)
                .retrieve()
               .onStatus(HttpStatus::is4xxClientError, clientResponse-> clientResponse.bodyToMono(Map.class).flatMap(e->{
                    log.error(" 4XX error is occurred ");
                    return Mono.error(new Exception());
                }))
                .onStatus(HttpStatus::is5xxServerError, serverResponse->serverResponse.bodyToMono(Map.class).flatMap(e->{
                    log.error("there is 5XX error ");
                    return Mono.error(new Exception());
                }))
                .bodyToFlux(Order.class);

    }

    @Override
    public Flux<Product> getAllProductsFromService() {
        return this.webClient.get().uri(url+ ClientUtils.PRODUCTS).retrieve()
                .onStatus(HttpStatus::is4xxClientError, clientResponse-> clientResponse.bodyToMono(Map.class).flatMap(e->{
                    log.error(" 4XX error is occurred ");
                    return Mono.error(new Exception());
                }))
                .onStatus(HttpStatus::is5xxServerError, serverResponse->serverResponse.bodyToMono(Map.class).flatMap(e->{
                    log.error("there is 5XX error ");
                    return Mono.error(new Exception());
                }))
                .bodyToFlux(Product.class);
    }
}
